# Algorithm
LSTM = 'lstm'
DENSE = 'dense'
PHASED_LSTM = 'phasedlstm'

# Inference type
ESTIMATOR = 'Estimator'
CLASSIFIER = 'Classifier'

# Data types
TIMESERIES = 'timeseries'

# Activation functions
LINEAR = 'linear'
SOFTMAX = 'softmax'
RELU = 'relu'
SIGMOID = 'sigmoid'

# Monitors
VAL_LOSS = 'val_loss'

# File extensions
MODEL_EXTENSION = 'h5'
SCALER_EXTENSION = 'pkl'
LABELENCODER_EXTENSION = 'e.pkl'
ONEHOTENCODER_EXTENSION = 'ohe.pkl'
JSON_EXTENSION = 'json'
PREDICTOR = 'predictor'
# Logging levels
DEBUG = 'debug'
INFO = 'info'
WARNING = 'warning'
ERROR = 'error'
PERFORMANCE = 'perf'